#!/usr/bin/python
#
# Parse CC2500 commands from Scana logic SPI traces (csv export)
#
#  Based upon idea of nRF24_decode.py from victz
# 

# Command strobes
cmd_strobe_map = {
0x30 : "SRES",
0x31 : "SFSTXON",
0x32 : "SXOFF",
0x33 : "SCAL",
0x34 : "SRX",
0x35 : "STX",
0x36 : "SIDLE",
0x38 : "SWOR",
0x39 : "SPWD",
0x3A : "SFRX",
0x3B : "SFTX",
0x3C : "SWORRST",
0x3D : "SNOP"
}

# Configuration registers
conf_reg_map = {
0x00 : "IOCFG2 ",
0x01 : "IOCFG1",
0x02 : "IOCFG0",
0x03 : "FIFOTHR",
0x04 : "SYNC1",
0x05 : "SYNC0",
0x06 : "PKTLEN",
0x07 : "PKTCTRL1",
0x08 : "PKTCTRL0",
0x09 : "ADDR",
0x0A : "CHANNR",
0x0B : "FSCTRL1",
0x0C : "FSCTRL0",
0x0D : "FREQ2",
0x0E : "FREQ1",
0x0F : "FREQ0",
0x10 : "MDMCFG4",
0x11 : "MDMCFG3",
0x12 : "MDMCFG2",
0x13 : "MDMCFG1",
0x14 : "MDMCFG0",
0x15 : "DEVIATN",
0x16 : "MCSM2",
0x17 : "MCSM1",
0x18 : "MCSM0",
0x19 : "FOCCFG",
0x1A : "BSCFG",
0x1B : "AGCTRL2",
0x1C : "AGCTRL1",
0x1D : "AGCTRL0",
0x1E : "WOREVT1",
0x1F : "WOREVT0",
0x20 : "WORCTRL",
0x21 : "FREND1",
0x22 : "FREND0",
0x23 : "FSCAL3",
0x24 : "FSCAL2",
0x25 : "FSCAL1",
0x26 : "FSCAL0",
0x27 : "RCCTRL1",
0x28 : "RCCTRL0",
0x29 : "FSTEST",
0x2A : "PTEST",
0x2B : "AGCTEST",
0x2C : "TEST2",
0x2D : "TEST1",
0x2E : "TEST0",
0x2F : "FOCCFG"
}

# Status registers
status_reg_map = {
0x30 : "PARTNUM",
0x31 : "VERSION",
0x32 : "FREQEST",
0x33 : "LQI",
0x34 : "RSSI",
0x35 : "MARCSTATE",
0x36 : "WORTIME1",
0x37 : "WORTIME0",
0x38 : "PKTSTATUS",
0x39 : "VCO_VC_DAC",
0x3A : "TXBYTES",
0x3B : "RXBYTES",
0x3C : "RCCTRL1_STATUS",
0x3D : "RCCTRL0_STATUS"
}


def dump_command(packet_id, command, status, data_out, data_in, timestamp):
  global bank
  global last_ts
  global first_ts
  global rf_counter
  if packet_id == -1: return
  mnemonic = "UNKNOWN"
  # Check if config register (0-0x2f)
  if (command & 0x3f) < 0x30:
      regnum = command & 0x3f;
      if (command & 0xc0) == 0x00:
         mnemonic = "WRITE_CFG(%s) " % conf_reg_map.get( regnum, " %02x " % regnum )
         mnemonic += " ".join(map(lambda x: "%02X" % x, data_out))
      elif (command & 0xc0) == 0x40:
         mnemonic = "WRITE_BURST_CFG(%s) " % conf_reg_map.get( regnum, " %02x " % regnum )
         mnemonic += " ".join(map(lambda x: "%02X" % x, data_out))
      elif (command & 0xc0) == 0x80:
         mnemonic = "READ_CFG(%s) " % conf_reg_map.get( regnum, " %02x " % regnum )
         mnemonic += " ".join(map(lambda x: "%02X" % x, data_in))
      elif (command & 0xc0) == 0xc0:
         mnemonic = "READ_BURST_CFG(%s) " % conf_reg_map.get( regnum, " %02x " % regnum )
         mnemonic += " ".join(map(lambda x: "%02X" % x, data_in))
  elif (command & 0x3f) < 0x3e:
      regnum = command & 0x3f;
      if (command & 0xc0) == 0x00:
         mnemonic = "WRITE_STROBE(%s) " % cmd_strobe_map.get( regnum, " %02x " % regnum )
         mnemonic += " ".join(map(lambda x: "%02X" % x, data_out))
      elif (command & 0xc0) == 0x40:
         mnemonic = "INVALID(%s) " % regnum
         mnemonic += " ".join(map(lambda x: "%02X" % x, data_out))
      elif (command & 0xc0) == 0x80:
         mnemonic = "READ_STROBE(%s) " % cmd_strobe_map.get( regnum, " %02x " % regnum )
         mnemonic += " ".join(map(lambda x: "%02X" % x, data_in))
      elif (command & 0xc0) == 0xc0:
         mnemonic = "READ_STATUS(%s) " % status_reg_map.get( regnum, " %02x " % regnum )
         mnemonic += " ".join(map(lambda x: "%02X" % x, data_in))
  else:
      if (command & 0x3f) == 0x3e:
         regname = "PATABLE"
      elif (command & 0xbf) == 0x3f:
         regname = "TX FIFO"
      else:
         regname = "RX FIFO"
      if (command & 0xc0) == 0x00:
         mnemonic = "WRITE(%s) " % regname
         mnemonic += " ".join(map(lambda x: "%02X" % x, data_out))
      elif (command & 0xc0) == 0x40:
         mnemonic = "WRITE_BURST(%s) " % regname
         mnemonic += " ".join(map(lambda x: "%02X" % x, data_out))
      elif (command & 0xc0) == 0x80:
         mnemonic = "READ(%s) " % regname
         mnemonic += " ".join(map(lambda x: "%02X" % x, data_in))
      elif (command & 0xc0) == 0xc0:
         mnemonic = "READ_BURST(%s) " % regname
         mnemonic += " ".join(map(lambda x: "%02X" % x, data_in))
    
  print "%3.3f %d %s" % (timestamp, packet_id, mnemonic)
  #print " %s" % (mnemonic)




def process_file(f):
    packet_id = -1
    # SPI exchange parameters
    command = -1
    status = -1
    data_out = []
    data_in = []
    timestamp = 0

    count = 0

    for line in f:
        line = line.strip()
        if len(line) == 0 or line[0] == '#': continue
        parts = line.split(',')
        if len(parts) != 4 or parts[1] == 'Packet ID' or parts[1] == '': continue
        
        # Cope with ASCII data, too
        # Note: This seems to be still incomplete
        if parts[2].startswith("0x"):
            dbout = int(parts[2], 16)
        elif len(parts[2]) == 1:
            dbout = ord(parts[2])
        elif parts[2] == "' '":
            dbout = 0x20
        elif parts[2].startswith("'"):
            dbout = int(parts[2].split("'")[1], 10)
        else:
            print "Unhandled format: line %d : part 2 = %s" % (count, parts[2])
            dbout = 0
        if parts[3].startswith("0x"):
            dbin = int(parts[3], 16)
        elif parts[3] == "' '":
            dbin = 0x20
        elif len(parts[3]) == 1:
            dbin = ord(parts[3])
        elif parts[3].startswith("'"):
            dbin = int(parts[3].split("'")[1], 10)
        else:
            print "Unhandled format: line %d : part 3 = %s" % (count, parts[3])
            dbin = 0
        
        if packet_id != int(parts[1]):
            # Skip status requests
            #if command != 0x07:
            dump_command(packet_id, command, status, data_out, data_in, timestamp)
            timestamp = float(parts[0])
            command = dbout;
            status = dbin;
            packet_id = int(parts[1])
            data_out = []
            data_in = []
        else:
            data_out.append(dbout)
            data_in.append(dbin)
        count = count +1
    dump_command(packet_id, command, status, data_out, data_in, timestamp)

def main(argv):
    for fn in argv:
        f = file(fn)
        process_file(f)

if __name__ == '__main__':
    import sys
    main(sys.argv[1:])

